This library is for reading and writing [JSON](https://www.json.org/). Because it has been created using progsbase, it is available in 12 programming languages and behaves the same in every one.

#### Quick Start - Java and Maven
Add the following to your POM:
```
<dependency>
    <groupId>com.progsbase.libraries</groupId>
    <artifactId>JSON</artifactId>
    <version>0.4.0</version>
</dependency>
```

You can read and write JSON to and from a class:
```
Example e = JSONReflectiveReader.readJSON(str, Example.class);
String str = JSONReflectiveWriter.writeJSON(e);
```

Or you can read or write JSON to and from a general Object, List<Object> and Map<String, Object>:
```
Object o = JSONObjectReader.readJSON(str);
String str2 = JSONObjectWriter.writeJSON(o);
```

Here is an example of reading a generic class:
```
List<List<Double>> e = JSONReflectiveReader.readJSON(
    "[[1.1, 2, 3], [1, 2, 3], [1, 2, 3]]", 
    List.class, 
    new GenericTypeGetter<List<List<Double>>>(){}.getType()
);
```

#### Quick Start - All languages
 * Goto "Download" below and select the programming language. Include the source in your project.
 * Goto [this example](.///JSONExample/) and select your language to see how to use the library.

#### About
JSON is a textual representation of a structure with maps and arrays.

The two main functions are [ReadJSON](.///ReadJSON), which reads JSON into an Data data structure, and [WriteJSON](.///WriteJSON), which writes an Data data structure to JSON. If there is a problem reading the JSON, the errorMessages array will contain one or more error messages.

 * [ReadJSON](.///ReadJSON)
 * [WriteJSON](.///WriteJSON)

The keys of the map must be strings. The values can be primitives, maps or arrays. An array can contain any combination of primitives, map and arrays.

The primitives supported by JSON are strings, numbers, booleans and null.

The JSON is read into general maps and arrays. Languages with reflection can put the elements of a map into the elements of a data structure with the same name. Languages with dynamic typing has native support for maps and arrays with mixed types. [Progsbase](https://progsbase.com) neither has reflection nor dynamic typing, so it reads the data into general maps and arrays. The values of the map and the contents of the arrays are elements that can either be one of the primitives, maps or arrays.

Some useful internal functions of this library are:
 * [IsValidJSON](.///IsValidJSON) - Checks whether a string is valid JSON.
 * [JSONCompare](.///JSONCompare) - Compares two JSON strings for equality. Two JSON strings are equal if they hold the same data according to the JSON standard.
 * [IsValidJSONString](.///IsValidJSONString) - Checks whether a string is a valid JSON string.
 * [IsValidJSONNumber](.///IsValidJSONNumber) - Checks whether a string is a valid JSON number.
 * [JSONTokenize](.///JSONTokenize) - Creates an array of valid JSON tokens.

#### Standards compliance
This implementation is compliant with the JSON standard [RFC 8259](https://tools.ietf.org/html/rfc8259).

It passes the following [test suite](https://github.com/nst/JSONTestSuite) titled "A comprehensive test suite for RFC 8259 compliant JSON parsers". This library accepts all the test cases which the author of the test suite says implementations are free to accept or reject.

#### Tests
Run the [tests](.///test) for the JavaScript version of the library directly in your browser now.
 * [tests](.///test)